using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;
using System.Collections;
using System.Net;
using System.Net.Sockets;

namespace AudioTool
{
    public partial class AudioToolForm : Form
    {
        #region class members

        public Capture applicationDevice = null;
        CaptureDevicesCollection devices = new CaptureDevicesCollection();
        public Guid CaptureDeviceGuid = Guid.Empty;
        public WaveFormat format;
        private ArrayList formats = new ArrayList();
        private bool[] InputFormatSupported = new bool[20];
        public WaveFormat InputFormat;
        public int CaptureBufferSize = 0;
        public int NotifySize = 0;


        public BufferPositionNotify[] PositionNotify = new BufferPositionNotify[NumberRecordNotifications + 1];
        public CaptureBuffer applicationBuffer = null;
        public const int NumberRecordNotifications = 16;
        public AutoResetEvent NotificationEvent = null;
        private string FileName = string.Empty;
        public Notify applicationNotify = null;
        private Thread NotifyThread = null;
        private FileStream WaveFile = null;
        private BinaryWriter Writer = null;
        private string Path = string.Empty;
        public int NextCaptureOffset = 0;
        private bool Recording = false;
        private int SampleCount = 0;
        private bool Capturing = false;
        private bool Playing = false;
        public float ampValue;

        Socket myUdp;
        IPEndPoint ep;

        public bool bRecording;
        private SecondaryBuffer playBuffer = null;
        private Device playDevice = null;
    
        private struct FormatInfo
        {
            public WaveFormat format;
            public override string ToString()
            {
                return ConvertWaveFormatToString(format);
            }
        };

        #endregion

        #region StartStopMain

        public AudioToolForm()
        {
            InitializeComponent();
        }

        [STAThread]
        public static void Main()
        {
            try
            {
                using (AudioToolForm form = new AudioToolForm())
                {
                    form.ShowDialog();
                }
            }
            catch { }
        }
    
        private void AudioToolForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (null != NotificationEvent)
            {
                Capturing = false;
                NotificationEvent.Set();
            }
        }

        private void AudioToolForm_Load(object sender, EventArgs e)
        {
            //Device
            foreach (DeviceInformation info in devices)
                devicesComboBox.Items.Add(info.Description);
            devicesComboBox.SelectedIndex = 0;
        }

        #endregion

        #region FormatStuff

        private void ScanAvailableInputFormats()
        {
            Console.WriteLine("Hey hey!");
            //-----------------------------------------------------------------------------
            // Name: ScanAvailableInputFormats()
            // Desc: Tests to see if 20 different standard wave formats are supported by
            //       the capture device 
            //-----------------------------------------------------------------------------
            WaveFormat format = new WaveFormat();
            CaptureBufferDescription dscheckboxd = new CaptureBufferDescription();
            CaptureBuffer pDSCaptureBuffer = null;

            // This might take a second or two, so throw up the hourglass
            Cursor = Cursors.WaitCursor;

            format.FormatTag = WaveFormatTag.Pcm;

            // Try 20 different standard formats to see if they are supported
            for (int iIndex = 0; iIndex < 20; iIndex++)
            {
                GetWaveFormatFromIndex(iIndex, ref format);

                // To test if a capture format is supported, try to create a 
                // new capture buffer using a specific format.  If it works
                // then the format is supported, otherwise not.
                dscheckboxd.BufferBytes = format.AverageBytesPerSecond;
                dscheckboxd.Format = format;

                try
                {
                    pDSCaptureBuffer = new CaptureBuffer(dscheckboxd, applicationDevice);
                    InputFormatSupported[iIndex] = true;
                }
                catch
                {
                    InputFormatSupported[iIndex] = false;
                }
                if (pDSCaptureBuffer != null)
                    pDSCaptureBuffer.Dispose();
            }
            Cursor = Cursors.Default;
        }
        
        private void GetWaveFormatFromIndex(int Index, ref WaveFormat format)
        {
            //-----------------------------------------------------------------------------
            // Name: GetWaveFormatFromIndex()
            // Desc: Returns 20 different wave formats based on Index
            //-----------------------------------------------------------------------------
            int SampleRate = Index / 4;
            int iType = Index % 4;

            switch (SampleRate)
            {
                case 0: format.SamplesPerSecond = 48000; break;
                case 1: format.SamplesPerSecond = 44100; break;
                case 2: format.SamplesPerSecond = 22050; break;
                case 3: format.SamplesPerSecond = 11025; break;
                case 4: format.SamplesPerSecond = 8000; break;
            }

            switch (iType)
            {
                case 0: format.BitsPerSample = 8; format.Channels = 1; break;
                case 1: format.BitsPerSample = 16; format.Channels = 1; break;
                case 2: format.BitsPerSample = 8; format.Channels = 2; break;
                case 3: format.BitsPerSample = 16; format.Channels = 2; break;
            }

            format.BlockAlign = (short)(format.Channels * (format.BitsPerSample / 8));
            format.AverageBytesPerSecond = format.BlockAlign * format.SamplesPerSecond;
        }
        
        void FillFormatListBox()
        {
            //-----------------------------------------------------------------------------
            // Name: FillFormatListBox()
            // Desc: Fills the format list box based on the availible formats
            //-----------------------------------------------------------------------------
            FormatInfo info = new FormatInfo();
            string strFormatName = string.Empty;
            WaveFormat format = new WaveFormat();

            for (int iIndex = 0; iIndex < InputFormatSupported.Length; iIndex++)
            {
                if (true == InputFormatSupported[iIndex])
                {
                    // Turn the index into a WaveFormat then turn that into a
                    // string and put the string in the listbox
                    GetWaveFormatFromIndex(iIndex, ref format);
                    info.format = format;
                    formats.Add(info);
                }
            }
            formatListBox.DataSource = formats;
        }
        
        private static string ConvertWaveFormatToString(WaveFormat format)
        {
            //-----------------------------------------------------------------------------
            // Name: ConvertWaveFormatToString()
            // Desc: Converts a wave format to a text string
            //-----------------------------------------------------------------------------
            return format.SamplesPerSecond + " Hz, " +
                format.BitsPerSample + "-bit " +
                ((format.Channels == 1) ? "Mono" : "Stereo");
        }

        #endregion

        #region Buttons
        private void devicesComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            CaptureDeviceGuid = devices[devicesComboBox.SelectedIndex].DriverGuid;
            InitDirectSound();
            //Format
            ScanAvailableInputFormats();
            FillFormatListBox();
            formatListBox.SelectedIndex = 8;
        }

        private void formatListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            InputFormat = ((FormatInfo)formats[formatListBox.SelectedIndex]).format;
            recordButton.Enabled = true;
            startButton.Enabled = true;
        }

        private void directoryButton_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                FileNameTextBox.Text = openFileDialog1.FileName;
        }
        
        private void startButton_Click(object sender, EventArgs e)
        {
            CreateCaptureBuffer();
            InitUdp();
            applicationBuffer.Start(true);
            stopButton.Enabled = true;
            startButton.Enabled = false;
            recordButton.Enabled = false;
            playButton.Enabled = false;
        }
    
        private void recordButton_Click(object sender, EventArgs e)
        {
            bRecording = true;
            CreateCaptureBuffer();
            OnCreateSoundFile();
            InitUdp();
            applicationBuffer.Start(true);
            stopButton.Enabled = true;
            recordButton.Enabled = false;
            startButton.Enabled = false;
            playButton.Enabled = false;
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            // Stop the buffer, and read any data that was not 
            // caught by a notification
            if (applicationBuffer!=null)
              applicationBuffer.Stop();

            if (playBuffer != null)
            {
                playBuffer.Stop();
                playBuffer.Dispose();
                playBuffer = null;
            }

            if (bRecording)
            {
                RecordCapturedData();
                Writer.Seek(4, SeekOrigin.Begin); // Seek to the length descriptor of the RIFF file.
                Writer.Write((int)(SampleCount + 36));	// Write the file length, minus first 8 bytes of RIFF description.
                Writer.Seek(40, SeekOrigin.Begin); // Seek to the data length descriptor of the RIFF file.
                Writer.Write(SampleCount); // Write the length of the sample data in bytes.

                Writer.Close();	// Close the file now.
                Console.WriteLine("File Closed:" + FileNameTextBox.Text);
                Writer = null;	// Set the writer to null.
                WaveFile = null; // Set the FileStream to null.OnCreateSoundFile();
            }
            bRecording = false;

            recordButton.Enabled = true;
            startButton.Enabled = true;
            playButton.Enabled = true;
            stopButton.Enabled = false;
        }
  
        private void playButton_Click(object sender, EventArgs e)
        {
            recordButton.Enabled = false;
            startButton.Enabled = false;
            playButton.Enabled = false;
            stopButton.Enabled = true;

            if (playDevice==null)
              playDevice = new Device();
            playDevice.SetCooperativeLevel(this, CooperativeLevel.Normal);
            try
            {
                playBuffer = new SecondaryBuffer(FileNameTextBox.Text, playDevice);
            }
            catch {  }
            
            if (playBuffer!=null)
              playBuffer.Play(0, BufferPlayFlags.Default);
        }

        #endregion

        #region Init Capturing

        private void InitDirectSound()
        {
            CaptureBufferSize = 0;
            NotifySize = 0;

            // Create DirectSound.Capture using the preferred capture device
            try
            {
                applicationDevice = new Capture(CaptureDeviceGuid);
            }
            catch { }
        }

        void CreateCaptureBuffer()
        {
            //-----------------------------------------------------------------------------
            // Name: CreateCaptureBuffer()
            // Desc: Creates a capture buffer and sets the format 
            //-----------------------------------------------------------------------------
            CaptureBufferDescription dscheckboxd = new CaptureBufferDescription();

            if (null != applicationNotify)
            {
                applicationNotify.Dispose();
                applicationNotify = null;
            }
            if (null != applicationBuffer)
            {
                applicationBuffer.Dispose();
                applicationBuffer = null;
            }

            if (0 == InputFormat.Channels)
                return;

            // Set the notification size
            NotifySize = (1024 > InputFormat.AverageBytesPerSecond / 32) ? 1024 : (InputFormat.AverageBytesPerSecond / 32);
            NotifySize -= NotifySize % InputFormat.BlockAlign;

            // Set the buffer sizes
            CaptureBufferSize = NotifySize * NumberRecordNotifications;

            // Create the capture buffer
            dscheckboxd.BufferBytes = CaptureBufferSize;
            InputFormat.FormatTag = WaveFormatTag.Pcm;
            dscheckboxd.Format = InputFormat; // Set the format during creatation

            applicationBuffer = new CaptureBuffer(dscheckboxd, applicationDevice);
            NextCaptureOffset = 0;

            InitNotifications();
        }
        
        void InitNotifications()
        {
            //-----------------------------------------------------------------------------
            // Name: InitNotifications()
            // Desc: Inits the notifications on the capture buffer which are handled
            //       in the notify thread.
            //-----------------------------------------------------------------------------

            if (null == applicationBuffer)
                throw new NullReferenceException();

            // Create a thread to monitor the notify events
            if (null == NotifyThread)
            {
                NotifyThread = new Thread(new ThreadStart(WaitThread));
                Capturing = true;
                NotifyThread.Start();

                // Create a notification event, for when the sound stops playing
                NotificationEvent = new AutoResetEvent(false);
            }


            // Setup the notification positions
            for (int i = 0; i < NumberRecordNotifications; i++)
            {
                PositionNotify[i].Offset = (NotifySize * i) + NotifySize - 1;
                PositionNotify[i].EventNotifyHandle = NotificationEvent.Handle;
            }

            applicationNotify = new Notify(applicationBuffer);

            // Tell DirectSound when to notify the app. The notification will come in the from 
            // of signaled events that are handled in the notify thread.
            applicationNotify.SetNotificationPositions(PositionNotify, NumberRecordNotifications);
        }

        #endregion

        private void WaitThread()
        {
            while (Capturing)
            {
                //Sit here and wait for a message to arrive
                if (NotificationEvent != null)
                {
                    NotificationEvent.WaitOne(Timeout.Infinite, true);
                    RecordCapturedData();
                }
            }
        }
        
        void RecordCapturedData()
        {
            //-----------------------------------------------------------------------------
            // Name: RecordCapturedData()
            // Desc: Copies data from the capture buffer to the output buffer 
            //-----------------------------------------------------------------------------
            byte[] CaptureData = null;
            int ReadPos;
            int CapturePos;
            int LockSize;

            applicationBuffer.GetCurrentPosition(out CapturePos, out ReadPos);
            LockSize = ReadPos - NextCaptureOffset;
            if (LockSize < 0)
                LockSize += CaptureBufferSize;

            // Block align lock size so that we are always write on a boundary
            LockSize -= (LockSize % NotifySize);

            if (0 == LockSize)
                return;

            // Read the capture buffer.
            CaptureData = (byte[])applicationBuffer.Read(NextCaptureOffset, typeof(byte), LockFlag.None, LockSize);

            //add amplitude bar
            ampValue=(float)CaptureData[CaptureData.Length - 1];
            
            
            // Write the data into the wav file
            if (bRecording)
            {
                Writer.Write(CaptureData, 0, CaptureData.Length);

                // Update the number of samples, in bytes, of the file so far.
                SampleCount += CaptureData.Length;

                // Move the capture offset along
                NextCaptureOffset += CaptureData.Length;
                NextCaptureOffset %= CaptureBufferSize; // Circular buffer
            }
        }

        #region WAV file stuff

        private void OnCreateSoundFile()
        {
            //-----------------------------------------------------------------------------
            // Name: OnCreateSoundFile()
            // Desc: Called when the user requests to save to a sound file
            //-----------------------------------------------------------------------------

            WaveFormat wf = new WaveFormat();
     
            // Get the default media path (something like C:\WINDOWS\MEDIA)
     
            FileName = FileNameTextBox.Text;
            try
            {
                CreateRIFF();
            }
            catch
            {
                return;
            }
        }
        
        void CreateRIFF()
        {
            /**************************************************************************
			 
                Here is where the file will be created. A
                wave file is a RIFF file, which has chunks
                of data that describe what the file contains.
                A wave RIFF file is put together like this:
			 
                The 12 byte RIFF chunk is constructed like this:
                Bytes 0 - 3 :	'R' 'I' 'F' 'F'
                Bytes 4 - 7 :	Length of file, minus the first 8 bytes of the RIFF description.
                                (4 bytes for "WAVE" + 24 bytes for format chunk length +
                                8 bytes for data chunk description + actual sample data size.)
                Bytes 8 - 11:	'W' 'A' 'V' 'E'
			
                The 24 byte FORMAT chunk is constructed like this:
                Bytes 0 - 3 :	'f' 'm' 't' ' '
                Bytes 4 - 7 :	The format chunk length. This is always 16.
                Bytes 8 - 9 :	File padding. Always 1.
                Bytes 10- 11:	Number of channels. Either 1 for mono,  or 2 for stereo.
                Bytes 12- 15:	Sample rate.
                Bytes 16- 19:	Number of bytes per second.
                Bytes 20- 21:	Bytes per sample. 1 for 8 bit mono, 2 for 8 bit stereo or
                                16 bit mono, 4 for 16 bit stereo.
                Bytes 22- 23:	Number of bits per sample.
			
                The DATA chunk is constructed like this:
                Bytes 0 - 3 :	'd' 'a' 't' 'a'
                Bytes 4 - 7 :	Length of data, in bytes.
                Bytes 8 -...:	Actual sample data.
			
            ***************************************************************************/

            // Open up the wave file for writing.
            WaveFile = new FileStream(FileName, FileMode.Create);
            Writer = new BinaryWriter(WaveFile);

            // Set up file with RIFF chunk info.
            char[] ChunkRiff = { 'R', 'I', 'F', 'F' };
            char[] ChunkType = { 'W', 'A', 'V', 'E' };
            char[] ChunkFmt = { 'f', 'm', 't', ' ' };
            char[] ChunkData = { 'd', 'a', 't', 'a' };

            short shPad = 1; // File padding
            int nFormatChunkLength = 0x10; // Format chunk length.
            int nLength = 0; // File length, minus first 8 bytes of RIFF description. This will be filled in later.
            short shBytesPerSample = 0; // Bytes per sample.

            // Figure out how many bytes there will be per sample.
            if (8 == InputFormat.BitsPerSample && 1 == InputFormat.Channels)
                shBytesPerSample = 1;
            else if ((8 == InputFormat.BitsPerSample && 2 == InputFormat.Channels) || (16 == InputFormat.BitsPerSample && 1 == InputFormat.Channels))
                shBytesPerSample = 2;
            else if (16 == InputFormat.BitsPerSample && 2 == InputFormat.Channels)
                shBytesPerSample = 4;

            // Fill in the riff info for the wave file.
            Writer.Write(ChunkRiff);
            Writer.Write(nLength);
            Writer.Write(ChunkType);

            // Fill in the format info for the wave file.
            Writer.Write(ChunkFmt);
            Writer.Write(nFormatChunkLength);
            Writer.Write(shPad);
            Writer.Write(InputFormat.Channels);
            Writer.Write(InputFormat.SamplesPerSecond);
            Writer.Write(InputFormat.AverageBytesPerSecond);
            Writer.Write(shBytesPerSample);
            Writer.Write(InputFormat.BitsPerSample);

            // Now fill in the data chunk.
            Writer.Write(ChunkData);
            Writer.Write((int)0);	// The sample length will be written in later.
        }

        #endregion

        private void InitUdp()
        {
            myUdp = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            IPAddress myAddr = IPAddress.Parse(ipTextBox.Text);
            int channel = 21840 + (int)channelSelect.Value;
            ep = new IPEndPoint(myAddr, channel);
        }

        private void amplitudeUpdate_Tick(object sender, EventArgs e)
        {
            
            if (playBuffer != null)
            {
                byte[] myData = null;
                int StartLoc, WriteLoc;
                
                playBuffer.GetCurrentPosition(out StartLoc, out WriteLoc);
                myData = (byte[])playBuffer.Read(StartLoc, typeof(byte), LockFlag.None, 4);
                ampValue = (float) myData[myData.Length-1];
            }

            if (format.BitsPerSample<16)
              amplitudeBar.Value = (int)Math.Abs((ampValue-128) / 128 * 100);
            else
              amplitudeBar.Value = (int)Math.Abs(ampValue / 255 * 100);
            
            ampText.Text = amplitudeBar.Value.ToString();
            
            byte[] ampBytes = new byte[1];
            ampBytes[0] = (byte)amplitudeBar.Value;
            
            if (amplitudeBar.Value > ThreshholdBar.Value)
            {
                if (myUdp != null)
                    myUdp.SendTo(ampBytes, ep);
            }
        }
    
    }
}