using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

using DirectShowLib;
using System.Runtime.InteropServices.ComTypes;

namespace MovieSandboxLib
{
    public class SimpleCaptureClass : ISampleGrabberCB, IDisposable
    {
          #region class variables
        public enum PlayState
        {
            Stopped,
            Paused,
            Running,
            Init
        };

    // Application-defined message to notify app of filtergraph events
        public const int WM_GRAPHNOTIFY = 0x8000 + 1;

        public IVideoWindow videoWindow = null;
        public IMediaControl mediaControl = null;
        IMediaEventEx mediaEventEx = null;
        IFilterGraph2 graphBuilder = null;
        public DsDevice device = null;
        ICaptureGraphBuilder2 captureGraphBuilder = null;
        public PlayState currentState = PlayState.Stopped;

        DsROTEntry rot = null;
      
        VideoWindow myVidWin;
        public int videoWidth;
        public int videoHeight;
        public int stride;
        public int count;

        public int threshhold;
        public Bitmap source;
        public int fluidSelect = 0;
        public Color myColor;
        public Color previewColor;

#endregion

    
    public SimpleCaptureClass(VideoWindow VidWin)
    {
        myVidWin = VidWin;
        myColor = Color.FromArgb(128, 128, 128);
        previewColor = Color.FromArgb(200, 0, 0);
    }

    #region BufferCallbacks

    int ISampleGrabberCB.SampleCB(double SampleTime, IMediaSample pSample)
    {
        
        Marshal.ReleaseComObject(pSample);
        return 0;
    }

    /// <summary> buffer callback, COULD BE FROM FOREIGN THREAD. </summary>
    int ISampleGrabberCB.BufferCB(double SampleTime, IntPtr pBuffer, int BufferLen)
    {
       
        lock (this)
            {
                source = new Bitmap(videoWidth, videoHeight, stride, PixelFormat.Format24bppRgb, pBuffer);
                for (int x = 0; x < videoWidth; x++)
                {
                    for (int y = 0; y < videoHeight; y++)
                    {
                        Color pointColor = source.GetPixel(x, y);
                        if (fluidSelect==0)
                        {
                            if ((int)pointColor.B + (int)pointColor.G + (int)pointColor.R < threshhold && threshhold > 0)
                            {
                                source.SetPixel(x, y, previewColor);
                            }
                        }
                        if (fluidSelect == 1)
                        {
                            if ((int)pointColor.B + (int)pointColor.G + (int)pointColor.R > threshhold && threshhold > 0)
                            {
                                source.SetPixel(x, y, previewColor);//Color.Black);
                            }
                        }
                    }
                }
            }
         return 0;
    }

        public void CopyToTarget(Bitmap target)
        {
            
            
        for (int x = 0; x < videoWidth; x++)
        {
            for (int y = 0; y < videoHeight; y++)
            {
                Color pointColor = source.GetPixel(x, y);
                if (pointColor == previewColor)
                {
                      target.SetPixel(x, y, myColor);
                }
            }
        }
        }

    #endregion
   
    #region Initialising Capture Stuff
  
    public void Dispose()
    {
       if (currentState==PlayState.Running)
        CloseInterfaces();
    }

    public void startCaptureVideoWindow(int num)
    {
        CaptureVideo(num);
    }

    public void CaptureVideo(int num)
    {
      int hr = 0;
      IBaseFilter sourceFilter = null;
//      IBaseFilter compressFilter = null;
      IBaseFilter grabFilter = null;
      ISampleGrabber sampGrabber = null;
      
      try
      {
        // Get DirectShow interfaces
        GetInterfaces();


       sourceFilter = FindCaptureDevice(num);
            
       // Add the capture device to the graph
       hr = this.graphBuilder.AddFilter(sourceFilter, "Video Device");
       DsError.ThrowExceptionForHR(hr);
      
       //// Get the SampleGrabber interface - only in playback!
       sampGrabber = (ISampleGrabber)new SampleGrabber();
       ConfigureSampleGrabber(sourceFilter, sampGrabber);
       grabFilter = (IBaseFilter)sampGrabber;

       hr = this.graphBuilder.AddFilter(grabFilter, "Grab Video");
       DsError.ThrowExceptionForHR(hr);

       hr = captureGraphBuilder.RenderStream(PinCategory.Preview, MediaType.Video, sourceFilter, grabFilter, null);
       DsError.ThrowExceptionForHR(hr);

       // Now that sizes are fixed, store the sizes
       SaveSizeInfo(sampGrabber);
       
          SetupVideoWindow();
 
        mediaControl = this.graphBuilder as IMediaControl;            

        // Now that the filter has been added to the graph and we have
        // rendered its stream, we can release this reference to the filter.

        if (sourceFilter != null)
        {
            Marshal.ReleaseComObject(sourceFilter);
            sourceFilter = null;
        }

        if (sampGrabber != null)
        {
            Marshal.ReleaseComObject(sampGrabber);
            sampGrabber = null;
        }
        
        if (grabFilter != null)
        {
            Marshal.ReleaseComObject(grabFilter);
            sampGrabber = null;
        }
 
        // Add our graph to the running object table, which will allow
        // the GraphEdit application to "spy" on our graph
        rot = new DsROTEntry(this.graphBuilder);

        // Start previewing video data
        hr = this.mediaControl.Run();
        DsError.ThrowExceptionForHR(hr);
 
        // Remember current state
        this.currentState = PlayState.Running;
      }
      catch
      {
        MessageBox.Show("An unrecoverable error has occurred.");
      }
   }

    private void ConfigureSampleGrabber(IBaseFilter capFilter, ISampleGrabber sampGrabber)
    {
        int hr;
        AMMediaType media = new AMMediaType();

        // Set the media type to Video/RBG24
        media.majorType = MediaType.Video;
        media.subType = MediaSubType.RGB24;
        media.formatType = FormatType.VideoInfo;
        hr = sampGrabber.SetMediaType(media);
        DsError.ThrowExceptionForHR(hr);

        DsUtils.FreeAMMediaType(media);
        media = null;

        // Configure the samplegrabber callback
        hr = sampGrabber.SetCallback(this, 1);
        DsError.ThrowExceptionForHR(hr);
    }
        
    private void SaveSizeInfo(ISampleGrabber sampGrabber)
    {
        int hr;

        // Get the media type from the SampleGrabber
        AMMediaType media = new AMMediaType();
        hr = sampGrabber.GetConnectedMediaType(media);
        DsError.ThrowExceptionForHR(hr);

        if ((media.formatType != FormatType.VideoInfo) || (media.formatPtr == IntPtr.Zero))
        {
            throw new NotSupportedException("Unknown Grabber Media Format");
        }

        // Grab the size info
        VideoInfoHeader videoInfoHeader = (VideoInfoHeader)Marshal.PtrToStructure(media.formatPtr, typeof(VideoInfoHeader));
        videoWidth = videoInfoHeader.BmiHeader.Width;
        videoHeight = videoInfoHeader.BmiHeader.Height;
        stride = videoWidth * (videoInfoHeader.BmiHeader.BitCount / 8);

        DsUtils.FreeAMMediaType(media);
        media = null;
    }

    public IBaseFilter FindCaptureDevice(int num)
    {
      DirectShowLib.DsDevice[] devices;
      object source;

      // Get all video input devices
      devices = DsDevice.GetDevicesOfCat(FilterCategory.VideoInputDevice);

      // Take the specified device
      device = devices[num];

        // Bind Moniker to a filter object
      Guid iid = typeof(IBaseFilter).GUID;
      device.Mon.BindToObject(null, null, ref iid, out source);
      
      return (IBaseFilter)source;
    }

    public void GetInterfaces()
    {
      int hr = 0;

      // An exception is thrown if cast fail
      this.graphBuilder = (IFilterGraph2) new FilterGraph();
      this.captureGraphBuilder = (ICaptureGraphBuilder2) new CaptureGraphBuilder2();
      this.mediaControl = (IMediaControl) this.graphBuilder;
      this.videoWindow = (IVideoWindow) this.graphBuilder;
      this.mediaEventEx = (IMediaEventEx) this.graphBuilder;

      hr = this.mediaEventEx.SetNotifyWindow(myVidWin.Handle, WM_GRAPHNOTIFY, IntPtr.Zero);
      DsError.ThrowExceptionForHR(hr);

      //Set the FilterGraph
      hr = captureGraphBuilder.SetFiltergraph(this.graphBuilder);
      DsError.ThrowExceptionForHR(hr);

    }

    public void CloseInterfaces()
    {
      // Stop previewing data
      if (this.mediaControl != null)
        this.mediaControl.StopWhenReady();

      this.currentState = PlayState.Stopped;

      // Stop receiving events
      if (this.mediaEventEx != null)
        this.mediaEventEx.SetNotifyWindow(IntPtr.Zero, WM_GRAPHNOTIFY, IntPtr.Zero);

      // Relinquish ownership (IMPORTANT!) of the video window.
      // Failing to call put_Owner can lead to assert failures within
      // the video renderer, as it still assumes that it has a valid
      // parent window.
      if(this.videoWindow != null)
      {
        this.videoWindow.put_Visible(OABool.False);
        this.videoWindow.put_Owner(IntPtr.Zero);
      }

      // Remove filter graph from the running object table
      if (rot != null)
      {
        rot.Dispose();
        rot = null;
      }

      // Release DirectShow interfaces
      Marshal.ReleaseComObject(this.mediaControl); this.mediaControl = null;
      Marshal.ReleaseComObject(this.mediaEventEx); this.mediaEventEx = null;
      Marshal.ReleaseComObject(this.videoWindow); this.videoWindow = null;
      Marshal.ReleaseComObject(this.graphBuilder); this.graphBuilder = null;
      Marshal.ReleaseComObject(this.captureGraphBuilder); this.captureGraphBuilder = null;
      myVidWin.Image = null;
      GC.Collect();
    }

    public void SetupVideoWindow()
    {
        int hr = 0;

        // Set the video window to be a child of the main window
        hr = this.videoWindow.put_Owner(myVidWin.Handle);
        DsError.ThrowExceptionForHR(hr);

        hr = this.videoWindow.put_WindowStyle(WindowStyle.Child | WindowStyle.ClipChildren);
        DsError.ThrowExceptionForHR(hr);

        // Use helper function to position video window in client rect 
        // of main application window
        ResizeVideoWindow();
    }

    public void ResizeVideoWindow()
    {
      // Resize the video preview window to match owner window size
      if (this.videoWindow != null)
      {
        //this.SetWindowForeground(true);
        this.videoWindow.SetWindowPosition(0, 0, myVidWin.ClientSize.Width, myVidWin.ClientSize.Height);
      }
    }

    public void ChangePreviewState(bool showVideo)
    {
      int hr = 0;

      // If the media control interface isn't ready, don't call it
      if (this.mediaControl == null)
        return;

      if (showVideo)
      {
        if (this.currentState != PlayState.Running)
        {
          // Start previewing video data
          hr = this.mediaControl.Run();
          this.currentState = PlayState.Running;
        }
      }
      else
      {
        // Stop previewing video data
        hr = this.mediaControl.StopWhenReady();
        this.currentState = PlayState.Stopped;
      }
    }

    public void HandleGraphEvent()
    {
      int hr = 0;
      EventCode evCode;
      int evParam1, evParam2;

      if (this.mediaEventEx == null)
        return;

      while(this.mediaEventEx.GetEvent(out evCode, out evParam1, out evParam2, 0) == 0)
      {
        // Free event parameters to prevent memory leaks associated with
        // event parameter data.  While this application is not interested
        // in the received events, applications should always process them.
        hr = this.mediaEventEx.FreeEventParams(evCode, evParam1, evParam2);
        DsError.ThrowExceptionForHR(hr);
        
        // Insert event processing code here, if desired
      }
    }
    
    protected void WndProc(ref Message m)
    {
      switch (m.Msg)
      {
        case WM_GRAPHNOTIFY:
        {
          HandleGraphEvent();
          break;
        }
      }

      // Pass this message to the video window for notification of system changes
      if (this.videoWindow != null)
        this.videoWindow.NotifyOwnerMessage(m.HWnd, m.Msg, m.WParam.ToInt32(), m.LParam.ToInt32());
  }

 #endregion

  public void pause()
  {
      mediaControl.Pause();
  }

  public void play()
  {
      mediaControl.Run();
  }

        public void stop()
        {
            mediaControl.StopWhenReady();
        }
 
    }
}
