using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using Microsoft.DirectX.DirectInput;
using WiimoteLib;

namespace Multi_HID_Input
{
    public partial class multiHID : Form
    {
        #region Variables

        public Device myJoy;
        public DeviceList myList;
        public byte[] hidData = new byte[32];
        Socket myUdp;
        IPEndPoint ep;

        //Wiimote
        private bool bWii=false;
        private Wiimote wm;
        public WiimoteState ws;
 
        //GameTrak
        public ushort xRot = 0;
        public ushort yRot = 0;
        public ushort zLength = 0;
        public ushort uRot = 0;
        public ushort vRot = 0;
        public ushort wLength = 0;

        public int directDevice=99;
        public int directChannel=99;

        #endregion
  
        public multiHID()
        {
            InitializeComponent();
        }
        
        public multiHID(string[] args)
        {
            if (args.Length > 0)
            {
                directDevice = Convert.ToInt16(args[0]);
                directChannel = Convert.ToInt16(args[1]);
            }
            InitializeComponent();
        }

        [STAThread]
        public static void Main(string[] args)
        {
            try
            {
                using (multiHID form = new multiHID(args))
                {
                    form.ShowDialog();
                }
            }
            catch { }
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            int devIndex=devicesListBox.SelectedIndex;   
         
            myList.Reset();
            myList.MoveNext();
            
            for (int i = 0; i < devIndex; i++)
                myList.MoveNext();
           
            DeviceInstance myJoyInput = (DeviceInstance)myList.Current;

            Guid myGuid = myJoyInput.InstanceGuid;
            myJoy = new Device(myGuid);
            if (myJoy.Properties.VendorIdentityProductId == 50726270)
            {
                ConnectWii();
                bWii = true;
            }
            else
                myJoy.Acquire();

            settingsButton.Enabled = true;
            InitUdp();
            udpSender.Start();
        }

        private void scanButton_Click(object sender, EventArgs e)
        {
            // clear List first
            devicesListBox.Items.Clear();
            //get HID from List
            myList = Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly);
            for (int i = 0; i < myList.Count; i++)
            {
                myList.MoveNext();
                devicesListBox.Items.Add(((DeviceInstance)myList.Current).ProductName);
            }

        }

        private void settingsButton_Click(object sender, EventArgs e)
        {
            if (myJoy != null)
            {
                myJoy.RunControlPanel();
            }

        }

        private void Gametrak_Tick()
        {
            if (myJoy != null)
            {
                //65536
                //1st Controller
                xRot = (ushort)myJoy.CurrentJoystickState.X;
                yRot = (ushort)myJoy.CurrentJoystickState.Y;

                zLength = (ushort)myJoy.CurrentJoystickState.Z;
                
                //2nd Controller
                uRot = (ushort)myJoy.CurrentJoystickState.Rx;
                vRot = (ushort)myJoy.CurrentJoystickState.Ry;

                wLength = (ushort)myJoy.CurrentJoystickState.Rz;

                dataLabelX.Text = Convert.ToString(xRot);
                dataLabelY.Text = Convert.ToString(yRot);
                dataLabelZ.Text = Convert.ToString(uRot);
                dataLabelRZ.Text = Convert.ToString(vRot);

                debugLabel.Text = Convert.ToString(zLength)+"  "+Convert.ToString(wLength);

                hidData[0] = (byte) (xRot >> 8);
                hidData[1] = (byte) xRot;

                hidData[2] = (byte)(yRot >> 8);
                hidData[3] = (byte)yRot;

                hidData[4] = (byte)(zLength >> 8);
                hidData[5] = (byte)zLength;

                hidData[6] = (byte)(uRot >> 8);
                hidData[7] = (byte)uRot;
            
                hidData[8] = (byte)(vRot >> 8);
                hidData[9] = (byte)vRot;
                
                hidData[10] = (byte)(wLength >> 8);
                hidData[11] = (byte)wLength;
            }
        }

        private void Joystick_Tick()
        {
            int pressedButton = 0;
            
            if (myJoy != null)
            {
                //dataLabelX.Text = myJoy.CurrentJoystickState.X.ToString();
                dataLabelX.Text = Convert.ToString(myJoy.CurrentJoystickState.X / 256);
                dataLabelY.Text = Convert.ToString(myJoy.CurrentJoystickState.Y / 256);
                dataLabelZ.Text = Convert.ToString(myJoy.CurrentJoystickState.Z / 256);
                //old joystick
                dataLabelRZ.Text = Convert.ToString(myJoy.CurrentJoystickState.Rz / 256);
                
                //new - for guitar Hero xplorer
                int[] pov = myJoy.CurrentJoystickState.GetPointOfView();
                //dataLabelRZ.Text = pov[0].ToString();
                if (pov[0] > 0)
                    pov[0] = 255;
                else if (pov[0] == 0)
                    pov[0] = 0;
                else if (pov[0] < 0)
                    pov[0] = 128;
               
                //dataLabelRZ.Text = pov[0].ToString();
                
                //hack for guitar hero controller 
                debugLabel.Text = pressedButton.ToString();

                hidData[0] = (byte)(myJoy.CurrentJoystickState.X / 256);
                hidData[1] = (byte)(myJoy.CurrentJoystickState.Y / 256);
                hidData[2] = (byte)(myJoy.CurrentJoystickState.Z / 256);
                hidData[3] = (byte)(myJoy.CurrentJoystickState.Rz / 256);
                //hidData[3] = (byte)(pov[0]);

                byte[] buttonState = myJoy.CurrentJoystickState.GetButtons();
                for (int i = 0; i < 27; i++)
                    hidData[i + 4] = buttonState[i];
            } 
        }

        private void Wiimote_Tick()
        {
            dataLabelX.Text = Convert.ToString((int) Math.Min((Math.Max(ws.AccelState.X+1,0) * 128),255));
            dataLabelY.Text = Convert.ToString((int) Math.Min((Math.Max(ws.AccelState.Y+1,0) * 128),255));
            dataLabelZ.Text = Convert.ToString((int) Math.Min((Math.Max(ws.AccelState.Z+1,0) * 128),255));

            debugLabel.Text = "A:" + Convert.ToString(Convert.ToInt16(ws.ButtonState.A)) + " B:" + Convert.ToString(Convert.ToInt16(ws.ButtonState.B)) + " 1:" + Convert.ToString(Convert.ToInt16(ws.ButtonState.One)) + " 2:" + Convert.ToString(Convert.ToInt16(ws.ButtonState.Two));

            //perform clamping
            if (128 + ws.AccelState.X * 128 < 256)
                hidData[0] = (byte)Math.Min((Math.Max(ws.AccelState.X+1,0) * 128),255);
            else
                hidData[0] = (byte)255;

            if (128 + ws.AccelState.Y * 128 < 256)
                hidData[1] = (byte)Math.Min((Math.Max(ws.AccelState.Y + 1, 0) * 128), 255);
            else
                hidData[1] = (byte)255;

            if (128 + ws.AccelState.Z * 128 < 256)
                hidData[2] = (byte)Math.Min((Math.Max(ws.AccelState.Z + 1, 0) * 128), 255);
            else
                hidData[2] = (byte)255;

            hidData[3] = (byte)(Convert.ToInt16(ws.ButtonState.A));
            hidData[4] = (byte)(Convert.ToInt16(ws.ButtonState.B));
            hidData[5] = (byte)(Convert.ToInt16(ws.ButtonState.One));
            hidData[6] = (byte)(Convert.ToInt16(ws.ButtonState.Two));
            hidData[7] = (byte)(Convert.ToInt16(ws.ButtonState.Left));
            hidData[8] = (byte)(Convert.ToInt16(ws.ButtonState.Right));
            hidData[9] = (byte)(Convert.ToInt16(ws.ButtonState.Up));
            hidData[10] = (byte)(Convert.ToInt16(ws.ButtonState.Down));
        }

        private void udpSender_Tick(object sender, EventArgs e)
        {
            if (myJoy.DeviceInformation.ProductName == "Game-Trak V1.3")
                Gametrak_Tick();
            else if (bWii)
                Wiimote_Tick();
            else 
                Joystick_Tick();

            myUdp.SendTo(hidData, ep);
        }

        private void InitUdp()
        {
            myUdp = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            IPAddress myAddr = IPAddress.Parse(ipTextBox.Text);
            int channel = 31840 + (int)channelSelect.Value;
            ep = new IPEndPoint(myAddr, channel);
        }

        private void ConnectWii()
        {
            // create a new instance of the Wiimote
            wm = new Wiimote();

            // setup the event to handle state changes
            wm.OnWiimoteChanged += new WiimoteChangedEventHandler(wm_OnWiimoteChanged);

            // setup the event to handle insertion/removal of extensions
            wm.OnWiimoteExtensionChanged += new WiimoteExtensionChanged(wm_OnWiimoteExtensionChanged);

            // connect to the Wiimote
            wm.Connect();
            // set the report type to return the IR sensor and accelerometer data (buttons always come back)
            wm.SetReportType(Wiimote.InputReport.IRAccel, true);

            wm.SetLEDs(true, false, false, false);
        }
        
        void wm_OnWiimoteExtensionChanged(object sender, WiimoteExtensionChangedEventArgs args)
        {
            if (args.Inserted)
                wm.SetReportType(Wiimote.InputReport.IRExtensionAccel, true);    // return extension data
            else
                wm.SetReportType(Wiimote.InputReport.IRAccel, true);            // back to original mode
        }

        void wm_OnWiimoteChanged(object sender, WiimoteChangedEventArgs args)
        {
            // current state information
            ws = args.WiimoteState;
        }

        private void multiHID_Load(object sender, EventArgs e)
        {
            
            scanButton_Click(this, null);
            if (directDevice <99 && directChannel < 99)
            {
                channelSelect.Value = directChannel;
                devicesListBox.SelectedIndex = directDevice;
                connectButton_Click(this, null);
            }
        }
    
    }
}