using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;


namespace MovieSandboxLib
{
    public partial class TracerApplication : Form
    {

        int resolution = 32;
        int boxX, boxY;
        int packetSize = 8;
        int minBlackLevel = 2;
        Color[] rgb = new Color[65537];
        Color[] zMap = new Color[65537];
        Color[] bMap = new Color[65537];

        Socket myUdp;
        IPEndPoint ep;

        [STAThread]
        public static void Main(string[] args)
        {
            try
            {
                using (TracerApplication form = new TracerApplication())
                {
                    form.ShowDialog();
                }
            }
            catch { }
        }

        public TracerApplication()
        {
            InitializeComponent();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
                ColorMap.ImageLocation=openFileDialog.FileName;
        }
        
        private void openHeightMapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                HeightMap.ImageLocation = openFileDialog1.FileName;
        
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Trace()
        {
            int row, line, picHeight, picWidth, value;
            Bitmap ColorBitmap, HeightBitmap, BoneBitmap;
            
            if (ColorMap.ImageLocation != "")
            {
                ColorBitmap = new Bitmap(ColorMap.ImageLocation);
                HeightBitmap = new Bitmap(HeightMap.ImageLocation);
                BoneBitmap = new Bitmap(BoneMap.ImageLocation);
            
                picHeight = ColorBitmap.Height;
                picWidth = ColorBitmap.Width;
                boxY = Height / resolution;


                for (line = 0; line < resolution; line++)
                {
                    for (row = 0; row < resolution; row++)
                    {
                        rgb[line * resolution + row] = ColorBitmap.GetPixel(line * picWidth / resolution, row * picHeight / resolution);
                        zMap[line * resolution + row] = HeightBitmap.GetPixel(line * picWidth / resolution, row * picHeight / resolution);
                        bMap[line * resolution + row] = BoneBitmap.GetPixel(line * picWidth / resolution, row * picHeight / resolution);
                    }
                }


                // re paint the image according to the trace


                for (line = 0; line < ColorBitmap.Width; line++)
                {
                    for (row = 0; row < ColorBitmap.Height; row++)
                    {
                        value = (line * resolution / ColorBitmap.Width) * resolution + row * resolution / ColorBitmap.Height;
                        if (value < resolution * resolution)
                        {
                            ColorBitmap.SetPixel(line, row, rgb[value]);
                            HeightBitmap.SetPixel(line, row, zMap[value]);
                            BoneBitmap.SetPixel(line, row, bMap[value]);
                        }
                    }
                }

                // reset the image
                ColorMap.Image = ColorBitmap;
                HeightMap.Image = HeightBitmap;
                BoneMap.Image = BoneBitmap;
                SendToMSBBtn.Enabled = true;
            }
        }

        private void myTick_Tick(object sender, EventArgs e)
        {
            int value=0;

            for (int i = 0; i < packetSize; i++)
            {
                if (boxY % resolution == 0)
                {
                    boxX--;
                    boxY = 0;
                }

                value = boxX * resolution - boxY;

     
                if (value > 0)
                    sendUDP(rgb[value].R, rgb[value].G, rgb[value].B, zMap[value].R, bMap[value].R, bMap[value].G, bMap[value].B);
                else
                {
                    myTick.Stop();
                    boxY = 0;
                    break;
                }
                boxY++;
            }
        
            progressBar1.Value = value;
        }


        private void InitUdp()
        {
            myUdp = new Socket(AddressFamily.InterNetwork, SocketType.Dgram,
                ProtocolType.Udp);
            IPAddress myAddr = IPAddress.Parse("127.0.0.1");
            ep = new IPEndPoint(myAddr, 41840);
        }



        private void sendUDP(byte R, byte G, byte B, byte A, byte BoneR, byte BoneG, byte BoneB)
        {
            byte[] sendArgs = new byte[7];
            if (R + G + B > minBlackLevel)
                sendArgs[0] = (byte)(resolution-1);

            sendArgs[1]=R;
            sendArgs[2]=G;
            sendArgs[3]=B;
            sendArgs[4]=A;
            sendArgs[5] = (byte)(resolution-1);
            sendArgs[6] = determineBone(BoneR,BoneG,BoneB);
         
            myUdp.SendTo(sendArgs,ep);
        }

        private byte determineBone(byte R, byte G, byte B)
        {
            if (R == 0 && B == 0 && G == 0)
                return 0;

            if (G == 64 && B == 0 && R == 0)
                return 5;
            if (G == 128 && B == 0 && R == 0)
                return 4;
            if (G == 255 && B == 0 && R == 0)
                return 3;
            if (B == 255 && G == 128 && R == 0)
                return 2;
            
            
            if (B == 255 && G == 0 && R == 128)
                return 6;
            if (R == 255 && B == 255 && G == 0)
                return 7;
            if (R == 128 && B == 128 && G == 0)
                return 8;
            if (R == 64 && B == 64 && G == 0)
                return 9;
                        
            
            if (R == 255 && B == 0 && G == 128)
                return 12;
            if (R == 255 && B == 0 && G == 0)
                return 13;
            if (R == 128 && B == 0 && G == 0)
                return 14;
            if (R == 64 && B == 0 && G == 0)
                return 15;


            if (B == 255 && R == 0 && G == 255)
                return 17;
            if (B == 128 && R == 0 && G == 128)
                return 18;
            if (B == 64 && R == 0 && G == 64)
                return 19;


            if (B == 255 && R == 0 && G == 0)
                return 21;
            if (B == 128 && R == 0 && G == 0)
                return 22;
            if (B == 64 && R == 0 && G == 0)
                return 23;
            
            if(G==255 && B==255 && R==255)
                return 25;

            return 0;
            
        }

        private void sendToMSB_Click(object sender, EventArgs e)
        {
            InitUdp();
            boxY = 0;
            boxX = resolution+1;
            progressBar1.Maximum = resolution * resolution;

            myTick.Start();
        }

        private void ColorMap_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
                ColorMap.ImageLocation = openFileDialog.FileName;
        }

        private void HeightMap_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                HeightMap.ImageLocation = openFileDialog1.FileName;
        }

        private void BoneMap_Click(object sender, EventArgs e)
        {
            if (openFileDialog2.ShowDialog() == DialogResult.OK)
                BoneMap.ImageLocation = openFileDialog2.FileName;
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            resolution = trackBar1.Value * 8;
            resBox.Text = trackBar1.Value.ToString();
            Trace();
        }

        private void TracerApplication_Load(object sender, EventArgs e)
        {
            ColorMap.ImageLocation = ".\\skel_color.jpg";
            HeightMap.ImageLocation = ".\\skel_height.jpg";
            BoneMap.ImageLocation = ".\\skel_bone.jpg";
        }

        private void resBox_TextChanged(object sender, EventArgs e)
        {
            if (resBox.Text != "" && Convert.ToInt16(resBox.Text) <= trackBar1.Maximum && Convert.ToInt16(resBox.Text) >= trackBar1.Minimum)
            {
                trackBar1.Value = Convert.ToInt16(resBox.Text);
                trackBar1_Scroll(sender, e);
            }
        }

        private void tickRateBox_TextChanged(object sender, EventArgs e)
        {
            if (tickRateBox.Text != "" && Convert.ToInt16(tickRateBox.Text) > 0)
                myTick.Interval = Convert.ToInt16(tickRateBox.Text);
        }

        private void packetBox_TextChanged(object sender, EventArgs e)
        {
            if (packetBox.Text != "" && Convert.ToInt16(packetBox.Text) > 0)
                packetSize = Convert.ToInt16(packetBox.Text);
        }

        private void blackLevelBox_TextChanged(object sender, EventArgs e)
        {
            if (blackLevelBox.Text != "" && Convert.ToInt16(blackLevelBox.Text) > 0)
                minBlackLevel = Convert.ToInt16(blackLevelBox.Text);
        }
        
    }
}