using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Diagnostics;
using System.Runtime.InteropServices;

using DirectShowLib;
using System.Runtime.InteropServices.ComTypes;

using System.Net;
using System.Net.Sockets;
using MovieSandboxLib;

namespace MovieSandboxLib
{
    public partial class VidCapWindow : Form
    {
        #region Class members

        KeyFrameButton[] myKeyFrames= new KeyFrameButton[25];
        int KeyCount = 0;
        bool bRecording = false;
        bool bSending = false;
        int dragBoneCount = 25;
        public MovieSandboxLib.draggablePictureBox[] myDragBonesFront = new draggablePictureBox[26];
        public MovieSandboxLib.draggablePictureBox[] myDragBonesSide = new draggablePictureBox[26];
        public byte[] dataSet = new byte[78];

        public int[] xOffset = new int[26];
        public int[] yOffset = new int[26];
        public int[] zOffset = new int[26];

        Socket myUdp;
        IPEndPoint ep;

        #endregion

        [STAThread]
        public static void Main(string[] args)
        {
            try
            {
                using (VidCapWindow form = new VidCapWindow())
                {
                    form.ShowDialog();
                }
            }
            catch { }
        }

        public VidCapWindow()
		{
			InitializeComponent();
  		}

        public void checkStatus()
        {
            if (videoWindow1.capLink.currentState == CaptureClass.PlayState.Running)
                videoWindow1.CloseInterfaces();
            if (videoWindow2.capLink.currentState == CaptureClass.PlayState.Running)
                videoWindow2.CloseInterfaces();
        }

        public void switchKeyFrame(int framePosition)
        {
            debugLabel.Text = "Debug:"+framePosition.ToString();
            timelineOne.Value = framePosition;
            timelineOne_Scroll(this, null);
        }

        private void frameTicker_Tick(object sender, EventArgs e)
        {
            timelineOne.Value = Convert.ToInt16(videoWindow1.capLink.getCurrentTime() * 25);
        }

        private void InitUdp()
        {
            myUdp = new Socket(AddressFamily.InterNetwork, SocketType.Dgram,
                ProtocolType.Udp);
            IPAddress myAddr = IPAddress.Parse("127.0.0.1");
            ep = new IPEndPoint(myAddr, 61840);
        }

        private void sendUDP()
        {
                myUdp.SendTo(dataSet, ep);
        }

        #region Button stuff
        private void startCaptureButton_Click(object sender, EventArgs e)
        {
            checkStatus();
            videoWindow1.capLink.startCaptureVideoWindow(videoInputBox1.SelectedIndex, fileName1.Text, false, false);
            videoWindow2.capLink.startCaptureVideoWindow(videoInputBox2.SelectedIndex, fileName2.Text, false, false);
            //videoWindow1.capLink.play();
            //videoWindow2.capLink.play();
            startPreviewButton.Text = "Stop Capture";
            bRecording = true;
        }

        private void startPreviewButton_Click(object sender, EventArgs e)
        {
            bRecording = false;
            startPreviewButton.Text = "Live Preview";
            checkStatus();
            videoWindow1.capLink.startCaptureVideoWindow(videoInputBox1.SelectedIndex, "previewOnly", true, false);
            videoWindow2.capLink.startCaptureVideoWindow(videoInputBox2.SelectedIndex, "previewOnly", true, false);
        }

        private void loadMovieButton_Click(object sender, EventArgs e)
        {
            checkStatus();
            videoWindow1.capLink.startCaptureVideoWindow(0, fileName1.Text, false, true);
            videoWindow2.capLink.startCaptureVideoWindow(1, fileName2.Text, false, true);

            //videoWindow1.capLink.pause();
            //videoWindow2.capLink.pause();
            timelineOne.Maximum = Convert.ToInt32(videoWindow1.capLink.getTotalTime() * 25);
            debugLabel.Text += "\n" + timelineOne.Maximum;
            totalTimeLabel.Text = "Total:"+videoWindow1.capLink.getTotalTime().ToString();
        }

        private void timelineOne_Scroll(object sender, ScrollEventArgs e)
        {
            if (videoWindow1.capLink.currentState != CaptureClass.PlayState.Stopped)
            {
                double position = timelineOne.Value;
                double duration = videoWindow1.capLink.getTotalTime();
                double absPosition = (position / 25);
                videoWindow1.capLink.jumpToPosition(absPosition);
               // videoWindow1.capLink.pause();
                
                duration = videoWindow2.capLink.getTotalTime();
                absPosition = (position/25);
                videoWindow2.capLink.jumpToPosition(absPosition);
             //   videoWindow2.capLink.pause();
                currentTimeLabel.Text = "Frame:" + timelineOne.Value.ToString();

                for (int i=0; i < KeyCount; i++)
                    if (timelineOne.Value == myKeyFrames[i].framePosition)
                        myKeyFrames[i].Checked = true;
            }
        }

        private void playBtnOne_Click(object sender, EventArgs e)
        {
            if (videoWindow1.capLink.currentState != CaptureClass.PlayState.Stopped)
            {
                videoWindow2.capLink.play();
                videoWindow1.capLink.play();
                if (!bRecording)
                frameTicker.Start();
            }
        }

        private void pauseBtnOne_Click(object sender, EventArgs e)
        {
            if (videoWindow1.capLink.currentState != CaptureClass.PlayState.Stopped)
            {
                videoWindow1.capLink.pause();
                videoWindow2.capLink.pause();
                frameTicker.Stop();
            }
        }

        private void addKeyFrameButton_Click(object sender, EventArgs e)
        {
            double a=timelineOne.Width;
            double b=timelineOne.Maximum;
            double c=timelineOne.Value;

            double KeyScreenPos = timelineOne.Left+ 9 + c * (a/b);
            
            //check if we don't spawn a second keyframe!!

            KeyFrameButton newKeyFrame = new KeyFrameButton();
            newKeyFrame.SetBounds(Convert.ToInt16(KeyScreenPos), timelineOne.Location.Y + 20, 15, 15);
            newKeyFrame.framePosition = timelineOne.Value;
            this.Controls.Add(newKeyFrame);
            newKeyFrame.Click += new System.EventHandler(newKeyFrame.KeyFrameButton_Click);
            
            myKeyFrames[KeyCount] = newKeyFrame;
            KeyCount++;
        }

        private void deleteKeyFrameButton_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < KeyCount; i++)
                if (myKeyFrames[i].Checked)
                {
                    myKeyFrames[i].Dispose();
                    for (int j=i;j<KeyCount;j++)
                      myKeyFrames[j]=myKeyFrames[j + 1];
                    KeyCount--;
                    break;
                }
        }

        private void sendUdpButton_Click(object sender, EventArgs e)
        {
            InitUdp();
            debugLabel.Text = "sending to moviesandbox...";
            bSending = true;
        }

        #endregion

        private void DrawLines()
        {
            System.Drawing.Pen myPen;
            System.Drawing.Graphics formGraphics = this.CreateGraphics();
            myPen = new System.Drawing.Pen(System.Drawing.Color.Red);
            formGraphics.Clear(this.BackColor);
            formGraphics.Clear(BackColor);          
            for (int i = 0; i < dragBoneCount; i++)
              if (myDragBonesFront[i]!=null && myDragBonesFront[i].ParentBone!=null)
                formGraphics.DrawLine(myPen, myDragBonesFront[i].Location.X, myDragBonesFront[i].Location.Y, myDragBonesFront[i].ParentBone.Location.X, myDragBonesFront[i].ParentBone.Location.Y);
            
            formGraphics.Dispose();
            myPen.Dispose();

            ////VideoWindow2
            myPen = new System.Drawing.Pen(System.Drawing.Color.Red);
            formGraphics = videoWindow2.CreateGraphics();
            for (int i = 0; i < dragBoneCount; i++)
             if (myDragBonesFront[i]!=null && myDragBonesFront[i].ParentBone != null)
               formGraphics.DrawLine(myPen, myDragBonesFront[i].Location.X, myDragBonesFront[i].Location.Y, myDragBonesFront[i].ParentBone.Location.X, myDragBonesFront[i].ParentBone.Location.Y);
           // formGraphics.DrawLine(myPen, 0, 0, 100, 100);
            formGraphics.Dispose(); 
            myPen.Dispose();
        }

        private void debugTicker_Tick(object sender, EventArgs e)
        {
            //fill boneArray
            int deltaX, deltaY, deltaZ;
            
            debugLabel.Text = "";
            
            for (int i = 0; i < myDragBonesFront.Length; i++)
                if (myDragBonesFront[i] != null && myDragBonesFront[i].ParentBone != null)
                {
                    //according to Unreal's coordinate system (Y is depth!):
                    deltaX = myDragBonesFront[i].ParentBone.Location.X - myDragBonesFront[i].Location.X +128;// -xOffset[i];
                    if (myDragBonesSide[i] != null)
                        deltaY = myDragBonesSide[i].Location.X - myDragBonesSide[i].ParentBone.Location.X +128;// - zOffset[i];
                    else
                        deltaY = 128;
                    deltaZ = myDragBonesFront[i].ParentBone.Location.Y - myDragBonesFront[i].Location.Y +128;// - yOffset[i];
                    
                    //draw info...
                    debugLabel.Text += "\n" + i.ToString() + ":" + myDragBonesFront[i].Name + ":     " + (deltaX - 128) + " " + (deltaY - 128) + " " + (deltaZ - 128);
                    
                    //store data
                    dataSet[i * 3 - 3] = (byte)(deltaX);
                    dataSet[i * 3 - 2] = (byte)(deltaY);
                    dataSet[i * 3 - 1] = (byte)(deltaZ);
                }

            if (bSending)
              sendUDP();

        //  DrawLines();
        }

        private void videoFormatBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (videoWindow1.capLink==null)
              videoWindow1.createCapLink();
            videoWindow1.capLink.vidRecordingCategory = FilterCategory.VideoInputDevice;
            videoWindow1.capLink.vidRecordingName = videoFormatBox1.SelectedItem.ToString();

            if (videoWindow2.capLink == null)
                videoWindow2.createCapLink();
            videoWindow2.capLink.vidRecordingCategory = FilterCategory.VideoInputDevice;
            videoWindow2.capLink.vidRecordingName = videoFormatBox1.SelectedItem.ToString();
        }

        private void VidCapWindow_Load(object sender, EventArgs e)
        {
            //enumerate Video Input filters and add them to comboBox1
            foreach (DsDevice ds in DsDevice.GetDevicesOfCat(FilterCategory.VideoInputDevice))
                videoInputBox1.Items.Add(ds.Name);

            foreach (DsDevice ds in DsDevice.GetDevicesOfCat(FilterCategory.VideoInputDevice))
                videoInputBox2.Items.Add(ds.Name);

            //enumerate Video Compressor filters and add them to comboBox1
            foreach (DsDevice ds in DsDevice.GetDevicesOfCat(FilterCategory.VideoCompressorCategory))
                videoFormatBox1.Items.Add(ds.Name);

            videoInputBox1.SelectedIndex = 0;
            videoInputBox2.SelectedIndex = 1;
            videoFormatBox1.SelectedIndex = 0;

            videoWindow1.capLink.vidRecordingCategory = FilterCategory.VideoInputDevice;
            videoWindow1.capLink.vidRecordingName = videoFormatBox1.SelectedItem.ToString();

            videoWindow2.capLink.vidRecordingCategory = FilterCategory.VideoInputDevice;
            videoWindow2.capLink.vidRecordingName = videoFormatBox1.SelectedItem.ToString();

            //set up Bones Front

            myDragBonesFront[1] = Pelvis_Front;
            myDragBonesFront[2] = Spine_Front;
            myDragBonesFront[3] = Spine1_Front;
            myDragBonesFront[4] = Spine2_Front;
            myDragBonesFront[5] = Neck_Front;

            // myDragBonesFront[6] = R_Clavicle_Front;  - not used!
            myDragBonesFront[7] = R_UpperArm_Front;
            myDragBonesFront[8] = R_ForeArm_Front;
            myDragBonesFront[9] = R_Hand_Front;
            // myDragBonesFront[10] = R_Finger_Front; - not used!
            // myDragBonesFront[11] = R_bone_weapon; - not used!

            // myDragBonesFront[12] = L_Clavicle_Side;  - not used!
            myDragBonesFront[13] = L_UpperArm_Front;
            myDragBonesFront[14] = L_ForeArm_Front;
            myDragBonesFront[15] = L_Hand_Front;
            // myDragBonesFront[16] = L_Finger_Side; - not used!

            myDragBonesFront[17] = R_Thigh_Front;
            myDragBonesFront[18] = R_Calf_Front;
            myDragBonesFront[19] = R_Foot_Front;
            // myDragBonesFront[20] = R_Toe0_Front; - not used!

            myDragBonesFront[21] = L_Thigh_Front;
            myDragBonesFront[22] = L_Calf_Front;
            myDragBonesFront[23] = L_Foot_Front;
            // myDragBonesFront[24] = R_Toe0_Front; - not used!

            myDragBonesFront[25] = Head_Front;

            //set up Bones Side

            myDragBonesSide[1] = Pelvis_Side;
            myDragBonesSide[2] = Spine_Side;
            myDragBonesSide[3] = Spine1_Side;
            myDragBonesSide[4] = Spine2_Side;
            myDragBonesSide[5] = Neck_Side;

            // myDragBonesSide[6] = R_Clavicle_Side;  - not used!
            myDragBonesSide[7] = R_UpperArm_Side;
            myDragBonesSide[8] = R_ForeArm_Side;
            myDragBonesSide[9] = R_Hand_Side;
            // myDragBonesSide[10] = R_Finger_Side; - not used!

     
            myDragBonesSide[17] = R_Thigh_Side;
            myDragBonesSide[18] = R_Calf_Side;
            myDragBonesSide[19] = R_Foot_Side;
            // myDragBonesSide[20] = R_Toe0_Side; - not used!

            myDragBonesSide[25] = Head_Side;

            //set up bone icons
            for (int i = 0; i < 26; i++)
            {
                //also, initialise offsets to zero
                xOffset[i] = 0;
                yOffset[i] = 0;
                zOffset[i] = 0;
                if (myDragBonesFront[i] != null)
                {
                    myDragBonesFront[i].ImageLocation = "..\\..\\PictureResource\\" + i.ToString() + ".jpg";
                    if (myDragBonesSide[i] != null)
                        myDragBonesFront[i].LinkedBone = myDragBonesSide[i];
                    myDragBonesFront[i].linkAxis = 1;
                }
            }
            for (int i = 0; i < 26; i++)
                if (myDragBonesSide[i] != null)
                {
                    myDragBonesSide[i].ImageLocation = "..\\..\\PictureResource\\" + i.ToString() + ".jpg";
                    myDragBonesSide[i].LinkedBone = myDragBonesFront[i];
                    myDragBonesSide[i].linkAxis = 1;
                }
            
            //set offsets
            //arms
            for (int i=7;i<16;i++)
                if (myDragBonesFront[i]!=null)
                    yOffset[i] = myDragBonesFront[i].ParentBone.Location.Y - myDragBonesFront[i].Location.Y;
            for (int i=17;i<24;i++)
                if (myDragBonesFront[i]!=null)
                    xOffset[i] = myDragBonesFront[i].ParentBone.Location.X - myDragBonesFront[i].Location.X;
        }



    }
}